# Schema.Org Converter (SDO-Converter)
This is a JavaScript tool to process the schema of [schema.org](http://schema.org/).

It takes the schema in JSON-LD format and transforms the data into several json files containing structured representations of the schema, its classes and properties. There is a [wiki](algorithm.md) explaining and justifying the algorithm and the generated data model of this tool. All data artifacts generated by this tool can be found in the "data_output" directory after it has been executed. 

## Prerequisites
* Have [Node.js](https://nodejs.org/en/) installed.
* Download latest version of the "schema.jsonld" from the [schema.org website](http://schema.org/docs/developers.html#formats) and copy the file in the "data_input" directory.
### Install
In the project's folder to download all dependencies:
```bash
npm install
```
### Usage
In the project's folder to start the tool:
```bash
node transform.js
```
### Output
The **data_output** directory will contain following files:

* **sdo_classes.json** - Contains the classes (types) of SDO with their characteristics. 
* **sdo_properties.json** - Contains the properties of SDO with their characteristics. 
* **sdo_dataTypes.json** - Contains the basic data types of SDO with their characteristics. 
* **sdo_enumerations.json** - Contains the enumeration classes of SDO with their characteristics. 
* **sdo_enumerationMembers.json** - Contains the enumeration instances of SDO with their characteristics. 
* **sdo_classesMaterialized.json** - Contains the classes (types), data types and enumerations of SDO with their characteristics. Some characteristics are materialized, eg. Class->properties or Enumeration->enumerationMembers. The materialization also includes inheritance for classes, hence all classes list the properties inherited from their superclasses.

There are also minified versions of the files (eg.: **sdo_classes.min.json**).

### Usage of data

The user can load this data in any way he wishes. However this project provides a small [library](sdo-converter-lib.js) to load and materialize the SDO data in the target application. The possibility to execute the materialization on the target application increases the performance of the application (load small data files, fast creation of materialized objects). This library is usable in browsers and in node.js.

Usage example for node.js:
```Javascript
//load library which creates the 'sdoConverter' singleton
require('../data/sdo-converter-lib.js')();

//load data and store it in memory of library
sdoConverter.set_memory_data('memory_classes', require('../data/sdo_classes.json'));
sdoConverter.set_memory_data('memory_properties', require('../data/sdo_properties.json'));
sdoConverter.set_memory_data('memory_dataTypes', require('../data/sdo_dataTypes.json'));
sdoConverter.set_memory_data('memory_enumerations', require('../data/sdo_enumerations.json'));
sdoConverter.set_memory_data('memory_enumerationMembers', require('../data/sdo_enumerationMembers.json'));

//start materialization 
sdoConverter.materialize_classesMaterialized();

//retrieve SDO data from library
let ClassesMaterializedObject = sdoConverter.get_allClassesMaterialized();
let PropertyObject_name = sdoConverter.get_property('name');
```

Usage example for browsers:

HTML
```HTML
<!-- load library which creates the 'sdoConverter' singleton -->
<script src="/assets/data/sdo-converter-lib.js"></script>
```
Javascript (incl. JQuery):
```Javascript
//start the load procedure
recursive_loadSDOFiles(0);

//process which loads the SDO files and starts the materialization
function recursive_loadSDOFiles(filePosition){
    if(sdoDataFiles.length > filePosition){
        var actualData = sdoDataFiles[filePos];
        $.getJSON("/assets/data/"+actualData.fileName, function(json){
            sdoConverter.set_memory_data(actualData.memoryName, json);
            recursive_loadSDOFiles(filePosition+1);
        });
    } else {
        //materialize classes
        sdoConverter.matergialize_classesMaterialized();
        //callback function
        callbackAfterLoading();
    }
}

//SDO files to load
var sdoDataFiles = [
    {
        fileName: 'sdo_classes.json',
        memoryName: 'memory_classes'
    },
    {
        fileName: 'sdo_properties.json',
        memoryName: 'memory_properties'
    },
    {
        fileName: 'sdo_dataTypes.json',
        memoryName: 'memory_dataTypes'
    },
    {
        fileName: 'sdo_enumerations.json',
        memoryName: 'memory_enumerations'
    },
    {
        fileName: 'sdo_enumerationMembers.json',
        memoryName: 'memory_enumerationMembers'
    }
];

//example callback function which is called after loading and materialization
function callbackAfterLoading() {
    //SDO data has been loaded and materialized at this point
    //library singleton sdoConverter can now be used from everywhere
    var SDOClasses =  sdoConverter.get_allClassesMaterialized();
    var dataType_Text = sdoConverter.get_dataType('Text');
}
```

### License

MIT © [Omar J. A. Holzknecht](http://nicarao.eu/)